from ..pyhornedowl import model
import typing

Class = model.Class
ObjectIntersectionOf = model.ObjectIntersectionOf
ObjectUnionOf = model.ObjectUnionOf
ObjectComplementOf = model.ObjectComplementOf
ObjectOneOf = model.ObjectOneOf
ObjectSomeValuesFrom = model.ObjectSomeValuesFrom
ObjectAllValuesFrom = model.ObjectAllValuesFrom
ObjectHasValue = model.ObjectHasValue
ObjectHasSelf = model.ObjectHasSelf
ObjectMinCardinality = model.ObjectMinCardinality
ObjectMaxCardinality = model.ObjectMaxCardinality
ObjectExactCardinality = model.ObjectExactCardinality
DataSomeValuesFrom = model.DataSomeValuesFrom
DataAllValuesFrom = model.DataAllValuesFrom
DataHasValue = model.DataHasValue
DataMinCardinality = model.DataMinCardinality
DataMaxCardinality = model.DataMaxCardinality
DataExactCardinality = model.DataExactCardinality
Datatype = model.Datatype
DataIntersectionOf = model.DataIntersectionOf
DataUnionOf = model.DataUnionOf
DataComplementOf = model.DataComplementOf
DataOneOf = model.DataOneOf
DatatypeRestriction = model.DatatypeRestriction
SimpleLiteral = model.SimpleLiteral
LanguageLiteral = model.LanguageLiteral
DatatypeLiteral = model.DatatypeLiteral
ObjectProperty = model.ObjectProperty
InverseObjectProperty = model.InverseObjectProperty
AnnotatedComponent = model.AnnotatedComponent
Annotation = model.Annotation
AnnotationAssertion = model.AnnotationAssertion
AnnotationProperty = model.AnnotationProperty
AnnotationPropertyDomain = model.AnnotationPropertyDomain
AnnotationPropertyRange = model.AnnotationPropertyRange
AnonymousIndividual = model.AnonymousIndividual
AsymmetricObjectProperty = model.AsymmetricObjectProperty
ClassAssertion = model.ClassAssertion
DataProperty = model.DataProperty
DataPropertyAssertion = model.DataPropertyAssertion
DataPropertyDomain = model.DataPropertyDomain
DataPropertyRange = model.DataPropertyRange
DatatypeDefinition = model.DatatypeDefinition
DeclareAnnotationProperty = model.DeclareAnnotationProperty
DeclareClass = model.DeclareClass
DeclareDataProperty = model.DeclareDataProperty
DeclareDatatype = model.DeclareDatatype
DeclareNamedIndividual = model.DeclareNamedIndividual
DeclareObjectProperty = model.DeclareObjectProperty
DifferentIndividuals = model.DifferentIndividuals
DisjointClasses = model.DisjointClasses
DisjointDataProperties = model.DisjointDataProperties
DisjointObjectProperties = model.DisjointObjectProperties
DisjointUnion = model.DisjointUnion
EquivalentClasses = model.EquivalentClasses
EquivalentDataProperties = model.EquivalentDataProperties
EquivalentObjectProperties = model.EquivalentObjectProperties
FacetRestriction = model.FacetRestriction
FunctionalDataProperty = model.FunctionalDataProperty
FunctionalObjectProperty = model.FunctionalObjectProperty
HasKey = model.HasKey
IRI = model.IRI
Import = model.Import
InverseFunctionalObjectProperty = model.InverseFunctionalObjectProperty
InverseObjectProperties = model.InverseObjectProperties
IrreflexiveObjectProperty = model.IrreflexiveObjectProperty
NamedIndividual = model.NamedIndividual
NegativeDataPropertyAssertion = model.NegativeDataPropertyAssertion
NegativeObjectPropertyAssertion = model.NegativeObjectPropertyAssertion
ObjectPropertyAssertion = model.ObjectPropertyAssertion
ObjectPropertyDomain = model.ObjectPropertyDomain
ObjectPropertyRange = model.ObjectPropertyRange
OntologyAnnotation = model.OntologyAnnotation
ReflexiveObjectProperty = model.ReflexiveObjectProperty
SameIndividual = model.SameIndividual
SubAnnotationPropertyOf = model.SubAnnotationPropertyOf
SubClassOf = model.SubClassOf
SubDataPropertyOf = model.SubDataPropertyOf
SubObjectPropertyOf = model.SubObjectPropertyOf
SymmetricObjectProperty = model.SymmetricObjectProperty
TransitiveObjectProperty = model.TransitiveObjectProperty
OntologyID = model.OntologyID
DocIRI = model.DocIRI
Rule = model.Rule
Variable = model.Variable
BuiltInAtom = model.BuiltInAtom
ClassAtom = model.ClassAtom
DataPropertyAtom = model.DataPropertyAtom
DataRangeAtom = model.DataRangeAtom
DifferentIndividualsAtom = model.DifferentIndividualsAtom
ObjectPropertyAtom = model.ObjectPropertyAtom
SameIndividualAtom = model.SameIndividualAtom
Facet = model.Facet
ClassExpression = model.ClassExpression
ObjectPropertyExpression = model.ObjectPropertyExpression
SubObjectPropertyExpression = model.SubObjectPropertyExpression
Literal = model.Literal
DataRange = model.DataRange
Individual = model.Individual
PropertyExpression = model.PropertyExpression
AnnotationSubject = model.AnnotationSubject
AnnotationValue = model.AnnotationValue
Component = model.Component
Atom = model.Atom
IArgument = model.IArgument
DArgument = model.DArgument

__all__ = ['Class', 'ObjectIntersectionOf', 'ObjectUnionOf', 'ObjectComplementOf', 'ObjectOneOf', 'ObjectSomeValuesFrom', 'ObjectAllValuesFrom', 'ObjectHasValue', 'ObjectHasSelf', 'ObjectMinCardinality', 'ObjectMaxCardinality', 'ObjectExactCardinality', 'DataSomeValuesFrom', 'DataAllValuesFrom', 'DataHasValue', 'DataMinCardinality', 'DataMaxCardinality', 'DataExactCardinality', 'Datatype', 'DataIntersectionOf', 'DataUnionOf', 'DataComplementOf', 'DataOneOf', 'DatatypeRestriction', 'SimpleLiteral', 'LanguageLiteral', 'DatatypeLiteral', 'ObjectProperty', 'InverseObjectProperty', 'AnnotatedComponent', 'Annotation', 'AnnotationAssertion', 'AnnotationProperty', 'AnnotationPropertyDomain', 'AnnotationPropertyRange', 'AnonymousIndividual', 'AsymmetricObjectProperty', 'ClassAssertion', 'DataProperty', 'DataPropertyAssertion', 'DataPropertyDomain', 'DataPropertyRange', 'DatatypeDefinition', 'DeclareAnnotationProperty', 'DeclareClass', 'DeclareDataProperty', 'DeclareDatatype', 'DeclareNamedIndividual', 'DeclareObjectProperty', 'DifferentIndividuals', 'DisjointClasses', 'DisjointDataProperties', 'DisjointObjectProperties', 'DisjointUnion', 'EquivalentClasses', 'EquivalentDataProperties', 'EquivalentObjectProperties', 'FacetRestriction', 'FunctionalDataProperty', 'FunctionalObjectProperty', 'HasKey', 'IRI', 'Import', 'InverseFunctionalObjectProperty', 'InverseObjectProperties', 'IrreflexiveObjectProperty', 'NamedIndividual', 'NegativeDataPropertyAssertion', 'NegativeObjectPropertyAssertion', 'ObjectPropertyAssertion', 'ObjectPropertyDomain', 'ObjectPropertyRange', 'OntologyAnnotation', 'ReflexiveObjectProperty', 'SameIndividual', 'SubAnnotationPropertyOf', 'SubClassOf', 'SubDataPropertyOf', 'SubObjectPropertyOf', 'SymmetricObjectProperty', 'TransitiveObjectProperty', 'OntologyID', 'DocIRI', 'Rule', 'Variable', 'BuiltInAtom', 'ClassAtom', 'DataPropertyAtom', 'DataRangeAtom', 'DifferentIndividualsAtom', 'ObjectPropertyAtom', 'SameIndividualAtom', 'Facet', 'ClassExpression', 'ObjectPropertyExpression', 'SubObjectPropertyExpression', 'Literal', 'DataRange', 'Individual', 'PropertyExpression', 'AnnotationSubject', 'AnnotationValue', 'Component', 'Atom', 'IArgument', 'DArgument']